;	
;	polarotor683_adj.asm	
;	Satellite Polarotor servo driver for 12F683
;	47k/10k voltage divider from coax to ADC	
;	GPIO:1 is coax voltage sense ADC input
;	GPIO:2 is center adjust via pot & ADC
;	Powers and sets skew of feedhorn servo
;	Bi color LED GPIO:4,5
;	Polarity debounce
;	Goal seek version
;	Brownout detect enabled
;	12/01/12
;	Functional
;------------------------------------------------------------
	LIST	P=12F683	
	ERRORLEVEL	-302
	include	<P12F683.INC>
	__CONFIG	h'33e4'
;------------------------------------------------------------	
;		Port Setup
;		+	+	+	+	+	+					
;	GPIO	0	1  	2	3       4       5
;		PWM    	Volts	Tweak	/MCLR	LED 	LED   
;------------------------------------------------------------
;	
	ORG	0
	goto	main
	ORG	4
	goto	interrupt_service
;------------------------------------------------------------
	cblock	0x20
	temp_W					;interrupt regs
	temp_S					;
;
	pw_low					;adding regs
	pw_high					;
	pwork_low				;working regs
	pwork_high				;
	volts_in				;AD results
	tweak_ctr				;AD in from 5v pot
	count					;
	rotor					;alternator
	debouncer				;
	delay					;delay timer	
        FLAGS                   		;various flags
;
 	endc
;------------------------------------------------------------
;
#define	PWM0_OUT	GPIO,0			;PWM0 output
;
#define	POL_FLAG	FLAGS,0			;0=V  1=H
#define	LED_FLAG	FLAGS,1			;LED state 
#define	H_TGT_FLAG	FLAGS,2			;set horz at target value
#define	V_TGT_FLAG	FLAGS,3			;set vert at target value
;			
SVO_V		equ	.1723		;servo +45 deg (assume tweak at 127)
;
SVO_H		equ	.1007		;servo -45 deg (assume tweak at 127)
;
SET4_VOLTS	equ	b'00000101'		;read voltage w/AD
SET4_ADJ	equ	b'00001001'		;read PW tweak w/AD
;
PULSE_GAP	equ	.65535 - .22000		;gap between pulses
;------------------------------------------------------------
;------------------------------------------------------------
main
	clrf	GPIO
	movlw	b'00000111'			;	
	movwf	CMCON0				;analogs off
	movlw	b'00000000'
	movwf	CMCON1				;no comparitor
	clrwdt	
	bsf	STATUS,RP0			;bank 1
	movlw	b'00001110'			;
	movwf	TRISIO				;CH5 PWM out
	movlw	b'01100000'
	movwf	OSCCON				;4 MHz int osc
	movlw	b'00000001'			;
	movwf	OPTION_REG			;TMR0/4, WPU enable
	movlw	b'00000010'
	movwf	WPU				;weak pullup select GP1
	movlw	b'00000000'
	movwf	IOC				;no int on change
	movlw	b'01010110'			;fosc/16, GP1,2 AD in
	movwf	ANSEL				;
	movlw	b'00000000'
	movwf	WPU				;no weak pullups
	movlw	b'00000001'			;
	movwf	PIE1				;TMR1 int enable
	bcf	STATUS,RP0			;bank 0
;
	movlw	b'00000101'			;L justify, vdd ref, on
	movwf	ADCON0				;
	movlw	b'00000001'
	movwf	T1CON				;timer /1, on
	movlw	b'00010110'			;max delay /64k
	movwf	WDTCON				;wdt off
;
	clrf	rotor				;
	clrf 	FLAGS				;reset
;
	movlw	b'01000000'
	movwf	INTCON				;PIE1 enable (TMR1)
	call	deb_delay			;let things settle
	call	grn_lite			;light green LED
;
;------------------------------------------------------------
;	
;	Reads GP:1 input w/debounce and sets flag to match
;	Starts in horz mode
;
start_here
	movlw	high SVO_H			;
	movwf	pwork_high			;
	movlw	low SVO_H			;
	movwf	pwork_low			;preload pulse width
;
	bsf	INTCON,GIE			;enable interrupt
;
sensor_loop
	call	deb_delay			;loop delay
	call	read_volts			;
	call	read_tweak			;
;
	movfw	volts_in			;
	sublw	.155				;high threshold (17.0v)
	bnc	hi_volts			;volts_in > 18
	movlw	.133				;low threshold (14.7v)
	subwf	volts_in,W			;
	skpnc
	goto	sensor_loop			;indetermined - leave as is
low_volts
	decf	debouncer,W			;
	xorlw	.255				;count down to zero
	bz	clear_pol_flag			;done clear flag
	decf	debouncer,F			;keep counting down
	goto	sensor_loop			;loop
;	
clear_pol_flag
	bcf	POL_FLAG			;volts low, make vert
	call	red_lite			;light red LED
	goto	sensor_loop			;loop
;
;
hi_volts
	incf	debouncer,W			;
	xorlw	.32				;count up to trigger point
	bz	set_pol_flag			;done set flag
	incf	debouncer,F			;keep counting up
	goto	sensor_loop			;loop
;
set_pol_flag
	bsf	POL_FLAG			;volts hi, horz
	call	grn_lite			;light green LED
	goto	sensor_loop			;loop
;
;============================================================
;						
interrupt_service
    	movwf   temp_W
	bcf	STATUS,RP0			;bank 0
    	swapf   STATUS,W
    	movwf   temp_S
	btfss	PIR1,TMR1IF			;insure TMR1 int
	goto	abort_int			;unknown int
	bcf	PIR1,TMR1IF			;clear flag
;
T1_handler
	bcf	PIR1,TMR1IF			;clear flag
	btfss	rotor,0				;alternating skip
	goto	no_pulse			;
	call	test_pol			;see if pw is changing
;
	movfw	pwork_high			;xfer working regs
	movwf	pw_high				;	
	movfw	pwork_low			;
	movwf	pw_low				;
;
	movfw	tweak_ctr			;center tweak
	addwf	pw_low,F			;added to pw_high, low
	skpnc					;
	incf	pw_high,F			;
;
        comf    pw_low,F			;invert results
        incf    pw_low,F			;allows for simple
        skpnz					;addition
        decf    pw_high,F
        comf    pw_high,F
;
	movfw	pw_low				; 
	addwf	TMR1L,F				;load pw to TMR1
	skpnc
	incf	TMR1H,F				;
	movfw	pw_high				;
	addwf	TMR1H,F
	bsf	PWM0_OUT			;pulse 0 high        
	goto	outta_here
;
no_pulse
	bcf	PWM0_OUT			;pulse goes low
	movlw	high PULSE_GAP
	movwf	TMR1H				;
	movlw	low PULSE_GAP
	movwf	TMR1L				;22 mS til next int
outta_here
	incf	rotor,F				;inc each cycle
abort_int
    	swapf   temp_S,W
    	movwf   STATUS
    	swapf   temp_W,F
    	swapf   temp_W,W
    	retfie                      		;return to program        
;
;------------------------------------------------------------
;               Subroutines
;------------------------------------------------------------
;
red_lite
	btfss	LED_FLAG
	return	
	bsf	GPIO,4				;
	bcf	GPIO,5				;red output
	bcf	LED_FLAG			;clear LED flag
	return
;
grn_lite
	btfsc	LED_FLAG
	return	
	bcf	GPIO,4				;
	bsf	GPIO,5				;green output
	bsf	LED_FLAG			;set LED flag
	return
;
;------------------------------------------------------------
;
;	POLFLAG=0, low volts, widening pulse is Vertical
;	POLFLAG=1, hi volts, narrowing pulse is Horizontal
;
;	pwork_high	pwork_low
;
test_pol
	btfss	POL_FLAG			;test polarity flag
	goto	do_vertical			;clear = vert
;
do_horz						;flag set = horiz
	bcf	V_TGT_FLAG			;clear opposite flag
	btfsc	H_TGT_FLAG			;Horz at target?
	return					;yes
;
	movlw	.16				;no, subtract 16
	subwf	pwork_low,F
	skpc					;pwork_low => .16
	decf	pwork_high,F
;
;if( K <= Y )
  	movlw 	high SVO_H
  	subwf 	pwork_high,w 			;
  	skpc
  	goto 	lock_horz			;pwork_high < high SVO_H
  	skpz
  	return					;pwork_high > high SVO_H
  	movlw 	low SVO_H			;
  	subwf 	pwork_low,w			;
	bz	lock_horz			;low SVO_H = pwork_low
  	skpnc					;low SVO_H > pwork_low ?
  	return
;
lock_horz	
	movlw	high SVO_H			;
	movwf	pwork_high			;
	movlw	low SVO_H			;
	movwf	pwork_low			;
	bsf	H_TGT_FLAG			;at target, set horz flag
	return
;
;
do_vertical
	bcf	H_TGT_FLAG			;clear opposite (horz)
	btfsc	V_TGT_FLAG			;vert at target?
	return					;yes
;
	movlw	.16				;no, add 16 and test
	addwf	pwork_low,F			;
	skpnc					;
	incf	pwork_high,F			;
;
;	if( pw_16 <= SVO_16 ) then horz_under_target
;
  	movfw 	pwork_high			;test hi byte
  	sublw 	high SVO_V 			;
  	skpc					;
  	goto 	lock_vert			;pwork_high > SVO_V
  	skpz					;	
  	return					;pwork_high < high SVO_V
  	movfw 	pwork_low			;pwork_high = high SVO_V
  	sublw 	low SVO_V			;do low bytes
	bz	lock_vert			;pwork_low = low SVO_V, on target
  	skpnc					;
	return					;pwork_low > low SVO_V
lock_vert
	movlw	high SVO_V			;
	movwf	pwork_high			;
	movlw	low SVO_V			;
	movwf	pwork_low			;
	bsf	V_TGT_FLAG			;at target, set flag
	return
;
;------------------------------------------------------------
;
read_tweak
	movlw	SET4_ADJ			;
	movwf	ADCON0				;read AN2
	call	aqu_delay			;wait
	bsf	ADCON0,GO			;start AD convert
;	
AD_loopA
	nop					;wait 1 instruct
	btfsc	ADCON0,GO
	goto	AD_loopA			;
;
	movfw	ADRESH				;msb 8 bits AD
	movwf	tweak_ctr			;store results
	return
;
;
read_volts
	movlw	SET4_VOLTS			;
	movwf	ADCON0				;read AN1
	call	aqu_delay			;wait
	bsf	ADCON0,GO			;start AD convert
;	
AD_loopB
	nop					;wait 1 instruct
	btfsc	ADCON0,GO
	goto	AD_loopB			;
;
	movfw	ADRESH				;msb 8 bits AD
	movwf	volts_in			;store results
	return
;
;------------------------------------------------------------
;	with TMR0/4 = 1 mS delay
deb_delay
	clrf	TMR0
	bcf	INTCON,T0IF
deb_loop
	btfss	INTCON,T0IF
	goto	deb_loop
	return
;
aqu_delay
	movlw	.10
	movwf	delay
aqu_delay_loop
	decfsz	delay,F
	goto	aqu_delay_loop
	return
;
;------------------------------------------------------------
	END
;	
;Copyright (c) 2012, Mike Berg
;All rights reserved.

;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:

;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;Neither the name of the <ringolake.com> nor the names of its 
;contributors may be used to endorse or promote products derived from 
;this software without specific prior written permission. 
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
;FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
;COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
;INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
;BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
;OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
;AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
;OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
;THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
;DAMAGE.
